import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {InvitesRepository} from './invites.repository';
import {InvitesEntity} from './entities/invites.entity';
import {CreateInviteDto} from './dto/createInvite.dto';
import {UsersRepository} from '../users/user.repository';
import {TransactionRepository} from '../transaction/transaction.repository';
import {PartyRepository} from '../party/party.repository';
import {invitationStatusType} from 'src/enum-type/invitation-status.enum';
import {UpdateInviteDto} from './dto/updateInvite.dto';
import { NotificationRepository } from "../notification/notification.repository";
import { CreateNotificationDto } from "../notification/dto/create-notification.dto";
import { notificationType } from "src/enum-type/notification.enum";

@Injectable()
export class InvitesService {
    constructor(
        @InjectRepository(InvitesRepository)
        private readonly invitesRepository: InvitesRepository,
        @InjectRepository(UsersRepository)
        private readonly usersRepository: UsersRepository,
        @InjectRepository(TransactionRepository)
        private readonly trasactionRepository: TransactionRepository,
        @InjectRepository(PartyRepository)
        private readonly partyRepository: PartyRepository,
        @InjectRepository(NotificationRepository)
        private readonly notificationRepository: NotificationRepository,
    ) {
    }

    async create(dto: CreateInviteDto, user: any): Promise<InvitesEntity | Error> {
        const {email} = dto;
        const invitedUser: any = await this.usersRepository.findByEmailAddress(email);
        if (!invitedUser) {
            throw new HttpException(
                `User not found with the provided email address!`,
                HttpStatus.NOT_FOUND
            );
        }
        const {id: invitedUserId} = invitedUser;
        const isUserpartyMember: any = await this.partyRepository.findByFilterQuery({
            userId: user.id,
            transactionId: dto.transactionId
        });
        if (isUserpartyMember.length <= 0) {
            throw new HttpException(
                `Unauthorized to invite!`,
                HttpStatus.BAD_REQUEST
            );
        }
        const isAlreadyPartyMember: any = await this.partyRepository.findByFilterQuery({
            userId: invitedUserId,
            transactionId: dto.transactionId
        });
        if (isAlreadyPartyMember.length > 0) {
            throw new HttpException(
                `Invited User is Already a Party Member!`,
                HttpStatus.BAD_REQUEST
            );
        }
        const isAlreadyInvitedMember: any = await this.invitesRepository.findByFilterQuery({
            invitedUserId: invitedUserId,
            transactionId: dto.transactionId,
            status: invitationStatusType.PENDING
        });
        if (isAlreadyInvitedMember.length > 0) {
            throw new HttpException(
                `Invited User is Already Invited!`,
                HttpStatus.BAD_REQUEST
            );
        }

        const createObj = {
            ...dto,
            invitedUserId: invitedUserId,
            invitedBy: user.id

        }
        const data = await this.invitesRepository.createEntity(createObj);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }

        const notificationDto: CreateNotificationDto = {
            userId: invitedUserId,
            transactionId: dto.transactionId,
            inviteId: (data as InvitesEntity).id,
            type: notificationType.INVITATION
        }
        this.notificationRepository.createEntity(notificationDto);

        return ResponseUtils.successResponseHandler(201, "Data Created Successfully", "data", data);
    }

    async findAll(user: any): Promise<InvitesEntity[] | Error> {
        const data = await this.invitesRepository.findByFilterQuery({
            invitedUserId: user.id,
            status: invitationStatusType.PENDING
        });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async findInviteDetails(id: string) {
        const data = await this.invitesRepository.findInviteDetails(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async findOne(id: string, user: any): Promise<InvitesEntity[] | Error> {
        const data: any = await this.invitesRepository.findByFilterQuery({
            invitedUserId: user.id,
            id: id,
            status: invitationStatusType.PENDING
        });
        if (data.length <= 0) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async update(id: string, dto: UpdateInviteDto, user: any): Promise<InvitesEntity | Error> {
        if (dto.status !== invitationStatusType.DECLINED && dto.status !== invitationStatusType.ACCEPTED) {
            throw new HttpException(
                `Not allowed to change to this invitation status!`,
                HttpStatus.BAD_REQUEST
            );
        }
        const findInvitation: any = await this.invitesRepository.findByFilterQuery({
            invitedUserId: user.id,
            id: id,
            status: invitationStatusType.PENDING
        });
        if (findInvitation.length <= 0) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const updateObj = {
            status: dto.status
        }
        const updatedInvite: any = await this.invitesRepository.updateEntity(id, updateObj);
        if (!updatedInvite) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        if (updatedInvite && dto.status === invitationStatusType.ACCEPTED) {
            const isAlreadyPartyMember: any = await this.partyRepository.findByFilterQuery({
                userId: user.id,
                transactionId: updatedInvite.transactionId
            });
            if (isAlreadyPartyMember.length > 0) {
                throw new HttpException(
                    `Invited User is Already a Party Member!`,
                    HttpStatus.BAD_REQUEST
                );
            }
            const partyObj = {
                transactionId: updatedInvite.transactionId,
                userId: user.id,
                userName: `${user.firstName} ${user.lastName}`

            }
            await this.partyRepository.createEntity(partyObj);
        }
        return ResponseUtils.successResponseHandler(200, "Data Updated Successfully", "data", updatedInvite);
    }

    async remove(id: string, user: any): Promise<boolean | Error> {
        const isInvitedByUser: any = await this.invitesRepository.findByFilterQuery({invitedBy: user.id, id: id});
        if (isInvitedByUser.length <= 0) {
            throw new HttpException(
                `You are unauthorized to performed this operation!`,
                HttpStatus.BAD_REQUEST
            );
        }
        if (isInvitedByUser[0].status !== invitationStatusType.PENDING) {
            throw new HttpException(
                `This invitation cannot be removed!`,
                HttpStatus.BAD_REQUEST
            );
        }
        const data = await this.invitesRepository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                `Failed to delete invitation`,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data Deleted Successfully");
    }
}
