import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { TransactionRepository } from './transaction.repository';
import { CreateTransactionDto } from './dto/create-transaction.dto';
import { UpdateTransactionDto } from './dto/update-transaction.dto';
import { TransactionEntity } from './entity/transaction.entity';
import { PartyRepository } from '../party/party.repository';
import { PaginationDto } from './dto/create-pagination.dto';
import { TransactionAddressDetailsRepository } from '../transaction-address-details/transaction-address-details.repository';
import { TransactionTimelineDetailsRepository } from '../transaction-timeline-details/transaction-timeline-details.repository';
import { LogsRepository } from '../transaction-logs/logs.repository';


@Injectable()
export class TransactionService {
    constructor(
        @InjectRepository(TransactionRepository)
        private readonly repository: TransactionRepository,
        @InjectRepository(PartyRepository)
        private readonly partyRepository: PartyRepository,
        @InjectRepository(TransactionAddressDetailsRepository)
        private readonly transactionAddressDetailsRepository: TransactionAddressDetailsRepository,
        @InjectRepository(TransactionTimelineDetailsRepository)
        private readonly transactionTimelineDetailsRepository: TransactionTimelineDetailsRepository,
        @InjectRepository(LogsRepository)
        private readonly logsRepository: LogsRepository,
    ) {
    }

    async create(dto: CreateTransactionDto, user: any): Promise<TransactionEntity | Error> {
        dto.userId = user.id;
        dto.userName = user.firstName + " " + user.lastName;
        const data: any = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        } else {
            const createPartyDto = {
                transactionId: data?.id,
                userId: dto.userId,
                userName: dto.userName
            }
            await this.partyRepository.createEntity(createPartyDto);
            await this.transactionAddressDetailsRepository.createEntity({
                transactionId: data?.id
            })
            await this.transactionTimelineDetailsRepository.createEntity({
                transactionId: data?.id
            })
            // create logs
            const createLogsDto = {
                userId: dto.userId,
                address: dto.streetAddress,
                clientName: dto.clientName,
                informationChnage: "Transaction created",
                reasonForChanging: null,
            }
            await this.logsRepository.createEntity(createLogsDto);

            return ResponseUtils.successResponseHandler(201, "Data Created Successfully", "data", data);
        }
    }

    async findAll(user: any): Promise<TransactionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ userId: user.id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async getTransactionReport(user: any): Promise<TransactionEntity[] | Error> {
        const transactionIds = await this.partyRepository.find({
            where: { userId: user.id, isDeleted: false },
            select: ["transactionId"]
        });

        const ids = transactionIds?.map(x => x.transactionId )

        const data = await this.repository.transactionReportQuery(ids);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    // async findAll(user: any, paginationDto: PaginationDto): Promise<Partial<TransactionEntity[]> | Error> {
    //     const { page, limit } = paginationDto;
    //     const offset = (page - 1) * limit;


    //     const data = await this.repository.findByFilterQuery({ userId: user.id }, offset, limit);

    //     if (!data) {
    //         throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
    //     }

    //     const totalCount = await this.repository.count({ userId: user.id });

    //     return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data, { page, limit, totalCount });
    // }

    async filterTransaction(user: any, paginationDto: PaginationDto): Promise<any | Error> {
        const data = await this.repository.filterTransaction(user.id, paginationDto);
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Transaction fetched successfully!',
            'transaction',
            data,
        );
    }


    async update(id: string, dto: UpdateTransactionDto, user: any): Promise<TransactionEntity | Error> {
        dto.userId = user.id;
        dto.userName = user.firstName + " " + user.lastName;
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const tran = await this.repository.findOneByFilterQuery({ id: id })
        // create logs
        const createLogsDto = {
            userId: tran[0].userId,
            address: tran[0].streetAddress,
            clientName: tran[0].clientName,
            informationChnage: "Updated transaction status",
            reasonForChanging: null,
        }
        await this.logsRepository.createEntity(createLogsDto);
        return ResponseUtils.successResponseHandler(200, "Data Updated Successfully", "data", data);
    }

    async findOne(id: string): Promise<TransactionEntity[] | Error> {
        const data = await this.repository.findOneByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const tran = await this.repository.findOneByFilterQuery({ id: id })
        // create logs
        const createLogsDto = {
            userId: tran[0].userId,
            address: tran[0].streetAddress,
            clientName: tran[0].clientName,
            informationChnage: "Delete a transaction",
            reasonForChanging: null,
        }
        await this.logsRepository.createEntity(createLogsDto);

        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.buildDeletedData(data, 200, "Data Deleted Successfully");
    }
}
