import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {ResponseUtils} from "src/utils/response.utils";
import {Constants} from "../../utils/constants";
import {FileUploadsService} from "../file-uploads/file-uploads.service";
import {SendEmailService} from "../send-email/send-email.service";
import {UploadedMulterFileI} from "../SpacesModule/SpacesService";
import {DoSpacesService} from "../SpacesModule/SpacesService/doSpacesService";
import {CreateUserDto} from "./dto/create-user.dto";
import {UpdateUserDto} from "./dto/update-user.dto";
import {User} from "./entities/user.entity";
import {UsersRepository} from "./user.repository";

@Injectable()
export class UsersService {
    constructor(
        @InjectRepository(UsersRepository)
        private readonly repository: UsersRepository,
        private readonly sendEmailService: SendEmailService,
        private readonly fileService: FileUploadsService,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(dto: CreateUserDto): Promise<User | Error> {
        const email = dto.email;

        const isEmailExists = await this.repository.findByEmailAddress(email);
        if (isEmailExists) {
            throw new HttpException("Email Already Exists", HttpStatus.BAD_REQUEST);
        }
        const data = await this.repository.createEntity(dto);

        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        if (dto.userType === "admin") {
            return ResponseUtils.successResponseHandler(
                201,
                "Admin created successfully!",
                "data",
                data
            );
        } else {
            return ResponseUtils.successResponseHandler(
                201,
                "User created successfully!",
                "data",
                data
            );
        }
    }

    async findAll(userType?: string): Promise<User[] | Error> {
        const data = await this.repository.findByQuery(userType);

        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        let userList = [];
        for (let i = 0; i < Object.keys(data).length; i++) {
            if (data[i].email !== "guest@goldengal.com") {
                userList.push(data[i]);
            }
        }
        return ResponseUtils.successResponseHandler(
            200,
            "Users data fetched!",
            "data",
            userList
        );
    }

    async findAllCommonTransactionUser(userId: string) {
        const data = await this.repository.findAllCommonTransactionUser(userId);

        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        return ResponseUtils.successResponseHandler(
            200,
            "Users data fetched!",
            "data",
            data
        );
    }

    async findCurrentTransactionUser(id: string, userType?: string, search?: string) {
        const data = await this.repository.findCurrentTransactionUser(id, userType, search);

        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        return ResponseUtils.successResponseHandler(
            200,
            "Users data fetched!",
            "data",
            data
        );
    }

    async findOne(id: string): Promise<User | Error> {
        const data = await this.repository.findOneEntity(id);

        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            "User data fetched!",
            "data",
            data
        );
    }

    async update(id: string, dto: UpdateUserDto): Promise<User | Error> {
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        return ResponseUtils.successResponseHandler(
            200,
            "User updated sucessfully!",
            "data",
            data
        );
    }

    async updateUserProfile(
        request: any,
        dto: UpdateUserDto,
        files: { image: UploadedMulterFileI }
    ): Promise<User | Error> {
        if (files && files.image) {
            dto.image = await this.doSpaceService.uploadFile(
                files.image[0],
                "profileImages/"
            );
        } else {
            dto["image"] = dto.image;
        }
        const data = await this.repository.updateEntity(request.user.id, dto);
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        return ResponseUtils.successResponseHandler(
            200,
            "User profile updated successfully!",
            "data",
            data
        );
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        return ResponseUtils.buildDeletedData(
            data,
            200,
            "User deleted successfully!"
        );
    }
}
